/*
 *  This file is part of "TweetyProject", a collection of Java libraries for
 *  logical aspects of artificial intelligence and knowledge representation.
 *
 *  TweetyProject is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright 2016 The TweetyProject Team <http://tweetyproject.org/contact/>
 */
 package net.sf.tweety.arg.aba.syntax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.tweety.arg.aba.semantics.AbaExtension;
import net.sf.tweety.arg.dung.syntax.Argument;
import net.sf.tweety.arg.dung.syntax.Attack;
import net.sf.tweety.arg.dung.syntax.DungTheory;
import net.sf.tweety.commons.BeliefBase;
import net.sf.tweety.commons.Formula;
import net.sf.tweety.commons.Signature;

/**
 * @author Nils Geilen <geilenn@uni-koblenz.de>
 * 
 * An implementation of Assumption Based Argumentation
 *
 * @param <T>
 *            is the type of the language that the ABA theory's rules range over
 */
public class ABATheory<T extends Formula> implements BeliefBase {

	/**
	 * The inference rules
	 */
	private Collection<InferenceRule<T>> rules = new HashSet<>();
	/**
	 * The assumptions used in this theory wehen no explicit set of assumptions
	 * is given
	 */
	private Collection<Assumption<T>> assumptions = new HashSet<>();
	
	/**
	 * The negation relation
	 */
	private Collection<Negation<T>> negations = new HashSet<>();

	/**
	 * @return all deductions that can be derived from this theory
	 */
	public Collection<Deduction<T>> getAllDeductions() {
		return getAllDeductions(assumptions);
	}

	/**
	 * @param assumptions
	 *            the set of assumptions used for the derivation
	 * @return all deductions that can be derived from this theory
	 */
	public Collection<Deduction<T>> getAllDeductions(Collection<Assumption<T>> assumptions) {		
		Set<Deduction<T>> args = new HashSet<>();
		for(ABARule<T> rule: this.rules)
			if(rule.isFact())
				args.add(new Deduction<T>("", rule));
		for(Assumption<T> a: assumptions)			
			args.add(new Deduction<T>("", a));
		boolean changed;
		do {
			changed = false;
			for(ABARule<T> rule: this.rules) {
				Collection<Collection<Deduction<T>>> subs = new HashSet<>();
				boolean continueWithNextRule = false;
				for(T prem: rule.getPremise()) {
					Collection<Deduction<T>> argsForPrem = new HashSet<>();					
					for(Deduction<T> arg: args)
						if(arg.getConclusion().equals(prem) && !arg.getAllConclusions().contains(rule.getConclusion())) 
							argsForPrem.add(arg);
					if(argsForPrem.isEmpty()) {
						continueWithNextRule = true;
						break;
					}else {
						if(subs.isEmpty()) {
							for(Deduction<T> subarg: argsForPrem) {							
								Collection<Deduction<T>> subargset = new HashSet<Deduction<T>>();
								subargset.add(subarg);								
								subs.add(subargset);
							}							
						}else {
							Collection<Collection<Deduction<T>>> new_subs = new HashSet<>();
							for(Deduction<T> subarg: argsForPrem) {
								for(Collection<Deduction<T>> s : subs) {
									Collection<Deduction<T>> newS = new HashSet<>(s);
									newS.add(subarg);
									new_subs.add(newS);
								}								
							}
							subs = new_subs;
						}
					}
				}
				if(continueWithNextRule)
					continue;
				for(Collection<Deduction<T>> subargset: subs)				
					changed = args.add(new Deduction<T>("",rule,subargset)) || changed;				
			}				
		}while(changed);
		return args;
	}

	/**
	 * A closure is the set of assumptions that can be derived from a set of
	 * assumptions via inference rules
	 * 
	 * @param assumptions
	 *            a set of assumptions
	 * @return the closure of assumptions
	 */
	public Collection<Assumption<T>> getClosure(Collection<Assumption<T>> assumptions) {
		Collection<Deduction<T>> deductions = getAllDeductions(assumptions);
		Set<Assumption<T>> cl = new HashSet<>();
		for (Assumption<T> assumption : this.assumptions) {
			for (Deduction<T> deduction : deductions) {
				if (assumption.getConclusion().equals(deduction.getConclusion())) {
					cl.add(assumption);
				}
			}
		}
		return cl;
	}

	/**
	 * A set of assumptions is closed iff it equals its closure
	 * 
	 * @param assumptions
	 *            a set of assumptions
	 * @return true iff the set of assumptions is closed under this
	 *         argumentation theory
	 */
	public boolean isClosed(Collection<Assumption<T>> assumptions) {
		Collection<Assumption<T>> cl = getClosure(assumptions);
		return cl.size() == assumptions.size() && cl.containsAll(assumptions) && assumptions.containsAll(cl);
	}

	/**
	 * An ABA theory is flat iff all subsets of its argumentation set are closed
	 * 
	 * @return true iff the theory is flat
	 */
	public boolean isFlat() {
		Collection<Collection<Assumption<T>>> powerset = toPowerSet(assumptions);
		for (Collection<Assumption<T>> asss : powerset) {
			if (!isClosed(asss))
				return false;
		}
		return true;
	}

	/**
	 * Computes the power set of a collection
	 * 
	 * @param set
	 *            a collection
	 * @return a power set
	 */
	private static <S> Collection<Collection<S>> toPowerSet(Collection<S> set) {
		Collection<Collection<S>> powerset = new HashSet<>();
		powerset.add(set);
		for (int i = 0; i < set.size(); i++) {
			List<S> list = new ArrayList<>(set);
			list.remove(i);
			powerset.addAll(toPowerSet(list));
		}
		return powerset;
	}

	/**
	 * @param rule
	 *            an assumption or an inference rule or a negation that is added to the theory
	 */
	@SuppressWarnings("unchecked")
	public void add(Formula rule) {
		if (rule instanceof Assumption)
			assumptions.add((Assumption<T>) rule);
		else if (rule instanceof InferenceRule)
			rules.add((InferenceRule<T>) rule);
		else if (rule instanceof Negation)
			negations.add((Negation<T>) rule);
	}

	/**
	 * @param assumption
	 *            a formula that is used as an assumption in the theory
	 */
	public void addAssumption(T assumption) {
		assumptions.add(new Assumption<>(assumption));
	}
	
	/**
	 * Adds a  negation of form not formula = negation
	 * @param formula	a formula
	 * @param negation	it's complement
	 */
	public void addNegation(T formula, T negation){
		negations.add(new Negation<>(formula, negation));
	}
	
	/**
	 * @param formula
	 * @param negation
	 * @return
	 */
	public boolean negates(T negation,T formula ){
		return negations.contains(new Negation<>(formula, negation));
	}
	
	/**
	 * @param atter	the attacking deduction
	 * @param atted	the attacked assumption
	 * @return	true iff atter attacks atted
	 */
	public boolean attacks(Deduction<T> atter, T atted){
		return negates(atter.getConclusion(),atted);
	}

	/**
	 * @return the rules
	 */
	public Collection<InferenceRule<T>> getRules() {
		return rules;
	}

	/**
	 * @return the assumptions
	 */
	public Collection<Assumption<T>> getAssumptions() {
		return assumptions;
	}

	/**
	 * @return the negations
	 */
	public Collection<Negation<T>> getNegations() {
		return negations;
	}

	/**
	 * @param assumptions
	 *            the assumptions to set
	 */
	public void setAssumptions(Collection<Assumption<T>> assumptions) {
		this.assumptions = assumptions;
	}
	
	public boolean attacks(Collection<Assumption<T>> atters, Collection<Assumption<T>> atteds) {
		for(Deduction<T> d : getAllDeductions(atters)) {
			for (Assumption<T> a : atteds) {
				if(negates(d.getConclusion(),a.getConclusion()))
					return true;
			}
		}
		return false;
	}
	
	/**
	 * Checks whether a set of arguments defends an argument
	 * @param defor	the defending set
	 * @param defed	the assumption which shall be defended
	 * @return	true iff defor defends defed
	 */
	public boolean defends(Collection<Assumption<T>> defor, Assumption<T> defed) {
		Collection<Assumption<T>> defedl = Arrays.asList(defed);
		for (Collection<Assumption<T>> ext:getAllExtensions()) {
			if(isClosed(ext)&&attacks(ext,defedl)&&!attacks(defor,ext))
				return false;
		}
		return true;
	}
	
	/**
	 * Checks whether a set of arguments is conflict-free
	 * @param ext	a set of arguments
	 * @return	true iff ext is conflict-free
	 */
	public boolean isConflictFree(Collection<Assumption<T>> ext) {
		return ! attacks(ext,ext);
	}
	
	/**
	 * Computes all possible extensions
	 * @return	the powerset of the assumptions
	 */
	public Collection<Collection<Assumption<T>>> getAllExtensions() {
		return toPowerSet(getAssumptions());
	}
	
	/**
	 * Computes all context-free extensions
	 * @return	all context-free extensions
	 */
	public Collection<Collection<Assumption<T>>> getAllConflictFreeExtensions() {
		Collection<Collection<Assumption<T>>>result = new HashSet<>();
		for(Collection<Assumption<T>> ext : toPowerSet(getAssumptions())){
			if(isConflictFree(ext))
				result.add(ext);
		}
		return result;
	}
	
	/**
	 * Checks whether a set of arguments is admissible
	 * @param ext	 the set
	 * @return	true iff ext is admissible
	 */
	public boolean isAdmissible(AbaExtension<T> ext){
		if(!isConflictFree(ext))
			return false;
		if(!isClosed(ext))
			return false;
		for(Collection<Assumption<T>> as : toPowerSet(getAssumptions())){
			if(isClosed(as)&&attacks(as, ext) && ! attacks(ext,as))
				return false;
		}
		return true;
	}
	
	/**
	 * Computes all admissible extensions
	 * @return	all admissible extensions
	 */
	public Collection<AbaExtension<T>> getAllAdmissbleExtensions() {
		Collection<AbaExtension<T>>result = new HashSet<>();
		for(Collection<Assumption<T>> ext : toPowerSet(getAssumptions())){
			AbaExtension<T> ext2 = new AbaExtension<T>(ext);
			if(isAdmissible(ext2))
				result.add(ext2);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.tweety.commons.BeliefBase#getSignature()
	 */
	@Override
	public Signature getSignature() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * @return a Dung Theory derived from this ABA theory
	 */
	public DungTheory asDungTheory() {
		if (!isFlat())
			throw new RuntimeException("Only flat ABA theories can be transformed into Dung theories.");
		Collection<Deduction<T>> ds = getAllDeductions();
		int id = 0;
		DungTheory dt = new DungTheory();
		Map<Deduction<T>, Argument> argmap = new HashMap<>();
		for (Deduction<T> d : ds) {
			Argument arg = d.getRule() instanceof Assumption<?> ? new Argument(d.getConclusion().toString())
					: new Argument("arg_" + id++);
			dt.add(arg);
			argmap.put(d, arg);
		}
		for (Deduction<T> atter : ds)
			for (Deduction<T> atted : ds)
				for (T ass : atted.getAssumptions())
					if (attacks(atter, ass)) {
						dt.add(new Attack(argmap.get(atter), argmap.get(atted)));
						break;
					}

		return dt;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "ABATheory [rules=" + rules + ", assumptions=" + assumptions + ", negations=" + negations + "]";
	}
	
	

}
